#include <stdlib.h>	// To prevent problems with missing built-in abs():
#include <math.h>
#include "counter.h"
#include "tools.h"
#include "names.h"
#include "exchange.h"
#include "cfg.h"

BlockCounter::BlockCounter() {
	_number=0;
}

int BlockCounter::set(int n) {
	if( (n < 0) || (n > 0xffff) ) return FAIL;
	_number = n;
	return SUCCESS;
}

int BlockCounter::incr(void) {
	if(_number >= 0xffff) _number = 0;
	else _number++;
	return _number;
}

int BlockCounter::decr(void) {
	if(_number <= 0) _number = 0xffff;
	else _number--;
	return _number;
}

Counter::Counter(void) {
	jdebug("counter::counter"); 
	_name = "Counters";
	resetVariables();
	jdebug("end of Counter's constructor");
}

Counter::~Counter() {
	delete[] data;
}

void Counter::resetVariables(void) {
	jdebug("counter::resetVariables");
	_levelValue = UNDEFINED;
	_expositionNumber = UNDEFINED;
	_exposValue = UNDEFINED;
	_maxNData = 0;
	blockCounter.reset();
	jdebug("End of counter's resetVariables");
}

void Counter::create(int sizeOfData) {
	_nData = 0;
	_maxNData = sizeOfData;
	data = new count_t[_maxNData];
	_nBlockLabel = 0;
	_blockSize = 0;
	blockLabel.resize(100);
}

void Counter::loadConst(void) {
	jdebug("counter::loadConst");
	_activeOn	= leafDataToChar( deviceCfgFile,_name,"Command","ActiveOn" );
	_activeOff	= leafDataToChar( deviceCfgFile,_name,"Command","ActiveOff" );
	_runMst		= leafDataToChar( deviceCfgFile,_name,"Command","Run" );
	_stop		= leafDataToChar( deviceCfgFile,_name,"Command","Stop" );
	_runTest	= leafDataToChar( deviceCfgFile,_name,"Command","RunTest" );
	_getNumber	= leafDataToChar( deviceCfgFile,_name,"Command","GetNumber" );
	_setNumber	= leafDataToChar( deviceCfgFile,_name,"Command","SetNumber" );
	_getBlSize	= leafDataToChar( deviceCfgFile,_name,"Command","GetBlSize" );
	_setBlSize	= leafDataToChar( deviceCfgFile,_name,"Command","SetBlSize" );
	_setLevel	= leafDataToChar( deviceCfgFile,_name,"Command","SetLevel" );
	_getLevel	= leafDataToChar( deviceCfgFile,_name,"Command","GetLevel" );
	_setExpos	= leafDataToChar( deviceCfgFile,_name,"Command","SetExpos" );
	_getExpos	= leafDataToChar( deviceCfgFile,_name,"Command","GetExpos" );
	_masterOn	= leafDataToChar( deviceCfgFile,_name,"Command","MasterOn" );
	_masterOff	= leafDataToChar( deviceCfgFile,_name,"Command","MasterOff" );
	_induceOn 	= leafDataToChar( deviceCfgFile,_name,"Command","InduceOn" );
	_induceOff	= leafDataToChar( deviceCfgFile,_name,"Command","InduceOff" );
	_setInduc	= leafDataToChar( deviceCfgFile,_name,"Command","SetInduc" );
	_getInduc	= leafDataToChar( deviceCfgFile,_name,"Command","GetInduc" );
	loadCommonConst();
	jdebug("End of counter's loadConst");
}

void Counter::resizeData(int newSize) {
	if(newSize > _maxNData) {
		_maxNData = newSize;
		delete[] data;
		data = new count_t[ _maxNData ];
	}
}

void Counter::addBlockLabel(int newLabel) {
	if(blockLabel.size() <= (uint)_nBlockLabel) {
		blockLabel.resize(blockLabel.size() + 100);
	}
	blockLabel[_nBlockLabel] = newLabel;
	_nBlockLabel++;
}

bool Counter::expandData(void) {
	int nCountInBlock = _blockSize / BYTE_PER_COUNT;
	if(!nCountInBlock) return false;
	int finiteLabel = _expositionNumber / nCountInBlock;

	bool isCorrected = false;
	for(int label=0; label<finiteLabel; label++) {
		if(label >= _nBlockLabel) {
			insertBlock(label, INVALID_COUNT);
			isCorrected = true;
			continue;
		}
		if(blockLabel[label] > label) { // skipped
			insertBlock(label,INVALID_COUNT);
			isCorrected = true;
			continue;
		}
		if(blockLabel[label] < label) { // multiplieds
			if(removeBlock(label) == 0) break;
			label--;
			continue;
		}
	}
	return isCorrected;
}

int Counter::repairData(void) {
	double avg_d = 0;
	int nValid = 0 ;
	for( int i=0; i<_nData; i++ ) {
		if( data[i] != INVALID_COUNT ) {
			avg_d += data[i];
			nValid++;
		}
	}
	if( nValid == 0 ) {
		// bad... nothing to divide for: return number of "corrected" = nData
		return _nData ;
	} 
	count_t avg = (count_t)rint( avg_d / nValid );

	// Replace INVALIDs with average:
	int nCorrected = 0;
	for(int i=0; i<_nData; i++) {
		if( data[i] == INVALID_COUNT ) {
			data[i] = avg ;
			nCorrected++;
		}
	}
	return nCorrected ;
}

int Counter::insertBlock(int label, count_t value) {
	int _nCountInBlock = _blockSize / BYTE_PER_COUNT;

	int start = 0;
	int startLabel = 0;
	for( int i=0; i<_nBlockLabel; i++ ) {
		if( blockLabel[i] == label-1 ) {
			start = (i+1)*_nCountInBlock;
			startLabel = i+1;
		}
	}
	if( (_nData+_nCountInBlock) > _maxNData ) {
		jdebug( "_nData+_nCountInBlock = %d > _maxNData = %d",_nData+_nCountInBlock,_maxNData );
		return(0);
	}
	_nData += _nCountInBlock;
	for( int j=(_nData-1); j>=(start+_nCountInBlock); j-- ) {
		data[j] = data[j-_nCountInBlock];
	}
	for( int j=start; j<(start+_nCountInBlock); j++ ) {
		data[j] = value;
	}

	_nBlockLabel++;
	for( int j=_nBlockLabel; j>startLabel; j-- ) {
		blockLabel[j] = blockLabel[j-1];
	}
	blockLabel[startLabel] = label;
	jdebug( "%s: block %d inserted",_identName,label );
	return(1);
}

int Counter::removeBlock(int label) {
	int start;
	int _nCountInBlock = _blockSize / BYTE_PER_COUNT;

	for( int i=0; i<_nBlockLabel; i++ ) {
		if( blockLabel[i] == label ) {
			start = i*_nCountInBlock;
			if( start+_nCountInBlock >= _nData ) {
				jdebug( "start+_nCountInBlock = %d > _nData = %d",start+_nCountInBlock, _nData );
				return(0); // not found
			}
			_nData -= _nCountInBlock;
			for( int j=start; j<_nData; j++ ) {
				data[j] = data[j+_nCountInBlock];
			}
			_nBlockLabel--;
			for( int j=i; j<_nBlockLabel; j++ ) {
				blockLabel[j] = blockLabel[j+1];
			}
			jdebug( "%s: block %d removed",_identName,label );
			return(1);
		}
	}
	jdebug( "block with label %d not found!", label );
	return(0); // not found
}

int Counter::putData(count_t dataVal) {
	if(_nData < _maxNData) {
		data[_nData] = dataVal;
		_nData++;
		return SUCCESS;
	}
	return FAIL;
}

int Counter::setOutNumber(int number) {
	if(number > _maxNData) return FAIL;
	_nData = number;
	return SUCCESS;
}

int Counter::searchTestErrors(count_t test_number) {
	_test_errors = 0;
	for(int i=0; i<_nData; i++) {
		test_number--;
		count_t derr = abs(data[i]-test_number);
		if( derr != 0 ) {
			_test_errors += derr; 
			test_number = data[i];
		}
	}
	_test_errors += data[_nData-1]; //yesli ne vse otshioty prishli
	return _test_errors;
}

int Counter::stop(int dialog=1) {
	jdebug("counter::stop");
	return sendCommand(_stop, dialog);
}

int Counter::setMasterOn(int dialog=1) {
	jdebug("counter::setMasterOn");
	return sendCommand(_masterOn, dialog);
}

int Counter::setMasterOff(int dialog=1) {
	jdebug("counter::setMasterOff");
	return sendCommand(_masterOff, dialog);
}

int Counter::setActiveOn(int dialog=1) {
	jdebug("counter::setActiveOn");
	return sendCommand(_activeOn, dialog);
}

int Counter::setActiveOff(int dialog=1) {
	jdebug("counter::setActiveOff");
	return sendCommand(_activeOff, dialog);
}

int Counter::setInduceOn(int dialog=1) {
	jdebug("counter::setInduceOn");
	return sendCommand(_induceOn, dialog);
}

int Counter::setInduceOff(int dialog=1) {
	jdebug("counter::setInduceOff");
	return sendCommand(_induceOff, dialog);
}

int Counter::setInduc(uchar masterAddress, int dialog=1) {
	jdebug("counter::setInduc");
	int current = UNDEFINED;
	return setValue(masterAddress, current, _setInduc, _getInduc,
						"Error in setInduc!", dialog);
}

int Counter::setBlSize(uchar blockSize, int dialog=1) {
	jdebug("counter::setBlSize");
	if( (blockSize > MAX_BLOCK_SIZE) || (blockSize < MIN_BLOCK_SIZE) ) {
		const char* errText = "Invalid block size!";
		reportError(_identName, errText, dialog);
		return FAIL;
	}
	return setValue(blockSize, _blockSize, _setBlSize, _getBlSize,
						"Error in setBlSize!", dialog);
}

int Counter::setNumber(int expositionNumber, int dialog=1) {
	jdebug("counter::setNumber");
	if( (expositionNumber > MAX_EXPOSITION_NUMBER) ||
			(expositionNumber < MIN_EXPOSITION_NUMBER) ) {
		const char* errText = "Invalid exposition number!";
		reportError(_identName, errText, dialog);
		return FAIL;
	}
	resizeData(expositionNumber);
	return setValue(expositionNumber,_expositionNumber,_setNumber,
						_getNumber, "Error in getNumber!", dialog);
}

int Counter::setLevel(double level, int dialog=1) {
	jdebug("counter::setLevel");

	int levelValue =
		int(rint(255+_const2-rint(level*(255.0+_const2-_const1))));

	if(levelValue < MIN_LEVEL_VALUE) levelValue = MIN_LEVEL_VALUE;
	if(levelValue > MAX_LEVEL_VALUE) levelValue = MAX_LEVEL_VALUE;

	return setValue(uchar(levelValue),_levelValue,_setLevel,_getLevel,
							"Error in setLevel!", dialog);
}

double Counter::getLevel(void) {
	return double(255.0+_const2-_levelValue)
					/ double(255.0+_const2-_const1);
}

int Counter::setExpos(double expos, int dialog=1) {
	jdebug("counter::setExpos");

	if( (expos < MIN_EXPOSE ) || (expos > MAX_EXPOSE ) ) {
		const char* errText = "Invalid exposure!";
		reportError(_identName, errText, dialog);
		return FAIL;
	}
	int exposValue = int(rint(expos*(_const3+(_const4<<8) - 1.0) / 8.0));

	return setValue(exposValue,_exposValue,_setExpos,_getExpos,
					"Error in setExpos!", dialog);
}

int Counter::runMst(int dialog=1) {
	jdebug("counter::runMst");
	return sendCommand(_runMst, dialog);
}

int Counter::runTest(int dialog=1) {
	jdebug("counter::runTest");
	return sendCommand(_runTest, dialog);
}
