#include <math.h>
#include <string.h>
#include "light.h"
#include "tools.h"
#include "names.h"
#include "exchange.h"
#include "cfg.h"

Light::Light(void) {
	jdebug("light::light");
	_name = "Lighters & Buttons";
	_identName = "Lighters & Buttons";
	_identSection = "Common";
	jdebug("end of light's constructor");
}

Light::~Light() {
}

void Light::resetVariables(void) {
	jdebug("Light::resetVariables");
	_illumLevel		= UNDEFINED;
	_illumValue 	= UNDEFINED;
	_illumination	= UNDEFINED;
	_light			= UNDEFINED;
	_lightValue		= UNDEFINED;
	_vary			= UNDEFINED;
	_vAmplValue		= UNDEFINED;
	_buttons		= UNDEFINED;
	_bMode			= UNDEFINED;
	jdebug("end of light's resetVariables");
}

void Light::loadConst(void) {
	jdebug("light: LoadConst");
	_setIllum	= leafDataToChar( deviceCfgFile,_name,"Command","SetIllum" );
	_getIllum	= leafDataToChar( deviceCfgFile,_name,"Command","GetIllum" );
	_setLight	= leafDataToChar( deviceCfgFile,_name,"Command","SetLight" );
	_getLight	= leafDataToChar( deviceCfgFile,_name,"Command","GetLight" );
	_setVAmpl	= leafDataToChar( deviceCfgFile,_name,"Command","SetVAmpl" );
	_getVAmpl	= leafDataToChar( deviceCfgFile,_name,"Command","GetVAmpl" );
	_setBMode	= leafDataToChar( deviceCfgFile,_name,"Command","SetBMode" );
	_getBMode	= leafDataToChar( deviceCfgFile,_name,"Command","GetBMode" );
	_setParam	= leafDataToChar( deviceCfgFile,_name,"Command","SetParam" );
	_getParam	= leafDataToChar( deviceCfgFile,_name,"Command","GetParam" );
	_illumOn	= leafDataToChar( deviceCfgFile,_name,"Command","IllumOn" );
	_illumOff	= leafDataToChar( deviceCfgFile,_name,"Command","IllumOff" );
	_lightOn	= leafDataToChar( deviceCfgFile,_name,"Command","LightOn" );
	_lightOff	= leafDataToChar( deviceCfgFile,_name,"Command","LightOff" );
	_varyOn		= leafDataToChar( deviceCfgFile,_name,"Command","VaryOn" );
	_varyOff	= leafDataToChar( deviceCfgFile,_name,"Command","VaryOff" );
	_buttLock	= leafDataToChar( deviceCfgFile,_name,"Command","ButtLock" );
	_buttUnLock	= leafDataToChar( deviceCfgFile,_name,"Command","ButtUnLock" );
	loadCommonConst();
	jdebug("end of light's loadConst");
}

int Light::testButtons(int dialog=1) {
	uchar status;
	if(getStatus(status,dialog) != SUCCESS) return FAIL;
	if(status & BUTT_LOCK) _buttons = 0;
	else _buttons = 1;
	return SUCCESS;
}

int Light::testIllum(int dialog=1) {
	uchar status;
	if(getStatus(status,dialog) != SUCCESS) return FAIL;
	if( status & ILLUM_ON ) _illumination = ON;
	else _illumination = OFF;
	if( status & LIGHT_ON ) _light = ON;
	else _light = OFF;
	return SUCCESS;
}

int Light::turnIllum(int state, int dialog=1) {	// state = ON or OFF
	jdebug("light::turnIllum");
	return turn(state,_illumination,_illumOn,_illumOff,dialog);
}

int Light::turnIllum(const char* state, int dialog=1) {
	if(strcasecmp(state,"ON") == 0) return turnIllum(ON,dialog);
	else return turnIllum(OFF,dialog);
}

int Light::turnLight(int state, int dialog=1,
			int waitTime=LIGHT_WAIT_INTERVAL) {
	jdebug("light::turnLight");
	if( _lightValue == 0 ) waitTime = 0; //    
	return turn(state,_light,_lightOn,_lightOff,dialog,waitTime);
}

int Light::turnLight(const char* state, int dialog=1,
			int waitInterval=LIGHT_WAIT_INTERVAL) {
	if(strcasecmp(state,"ON") == 0) return turnLight(ON,dialog,waitInterval);
	else return turnLight(OFF,dialog,waitInterval);
}

int Light::turnButtons(int state, int dialog=1) {
	jdebug("light::turnButtons");
	return turn(state,_buttons,_buttUnLock,_buttLock,dialog);
}

int Light::turnButtons(const char* state,int dialog=1) {
	if(strcasecmp(state,"ON") == 0) return turnButtons(ON,dialog);
	else return turnButtons(OFF,dialog);
}

int Light::turnVary(int state, int dialog=1) {
	jdebug("light::turnVary");
	return turn(state,_vary,_varyOn,_varyOff,dialog);
}

int Light::setBrightness(int illumLevel,int dialog=1) {
	if( illumLevel < MIN_ILLUM_LEVEL ) illumLevel = MIN_ILLUM_LEVEL;
	if( illumLevel > MAX_ILLUM_LEVEL ) illumLevel = MAX_ILLUM_LEVEL;
	if( illumLevel == _illumLevel ) return SUCCESS;

	int illumValue;
	if( illumLevel == 0 ) illumValue = 0;
	else illumValue = (uchar)floor( pow( 2.0,
				(8.0*illumLevel / 15.01) ) + 0.5 );
	jdebug("light::setBrightness:%d",illumValue );

	if( setValue(uchar(illumValue),_illumValue,_setIllum,_getIllum,
			"Error in setIllum!",dialog) != SUCCESS ) return FAIL;

	_illumLevel = illumLevel;
	return SUCCESS;
}

int Light::setBrightnessUp(int dialog=1) {
	if( _illumLevel >= MAX_ILLUM_LEVEL ) return SUCCESS;
	int illumLevel = _illumLevel+1;
	return setBrightness(illumLevel,dialog);
}

int Light::setBrightnessDown(int dialog=1) {
	if( _illumLevel <= MIN_ILLUM_LEVEL ) return SUCCESS;
	int illumLevel = _illumLevel-1;
	return setBrightness(illumLevel,dialog);
}

int Light::setLight(double lightLevel,int dialog=1,	//	lightLevel=[0.0..1.0]
				int waitTime=LIGHT_WAIT_INTERVAL) {
	if( lightLevel < MIN_LIGHT_LEVEL ) lightLevel = MIN_LIGHT_LEVEL;
	if( lightLevel > MAX_LIGHT_LEVEL ) lightLevel = MAX_LIGHT_LEVEL;

	uchar lightValue = uchar(doubleTo255int(lightLevel));
	if(_light != ON) waitTime = 0;	//  ,   
	return setValue(lightValue,_lightValue,_setLight,_getLight,
								"Error in setLight!",dialog,waitTime);
}

int Light::setVAmpl(double vAmplLevel,int dialog=1) {	//	vAmpl=[0.0..1.0]

	//   vary light

	if( (vAmplLevel < MIN_VAMPL_LEVEL ) || (vAmplLevel > MAX_VAMPL_LEVEL ) ) {
		const char* errText = "Invalid vAmpl level!";
		reportError(_identName,errText,dialog);
		return FAIL;
	}

	uchar vAmplValue = uchar(doubleTo255int(vAmplLevel));
	return setValue(vAmplValue,_vAmplValue,_setVAmpl,_getVAmpl,
								"Error in setVAmpl!",dialog);
}

int Light::getBMode(uchar& bMode,int dialog=1) {
	jdebug("light::getBMode");

	if( getValue(bMode,_getBMode,dialog) != SUCCESS ) return FAIL;

	if(bMode > MAX_BMODE) {
		const char* errText = "Invalid buttons mode!";
		reportError(_identName,errText,dialog);
		return FAIL;
	}

	bMode = bMode % N_BMODES;
	jdebug("BMODE = %d",bMode );
	return SUCCESS;
}

int Light::setBMode(uchar bMode,int dialog=1) {
	jdebug("light::setBMode");
	if(bMode > MAX_BMODE) {
		const char* errText = "Invalid buttons mode!";
		reportError(_identName,errText,dialog);
		return FAIL;
	}

	return setValue(bMode,_bMode,_setBMode,_getBMode,
								"Error in setBMode!",dialog);
}

int Light::getBParam(uchar& bParam,int dialog=1) {
	jdebug("light::getParam");
	return getValue(bParam,_getParam,dialog);
}

int Light::setBParam(uchar bParam,int dialog=1) {
	jdebug("light::setBParam");
	int oldParam = UNDEFINED;	//   
	return setValue(bParam,oldParam,_setParam,_getParam,
								"Error in setBParam!",dialog);
}
