#include <math.h>
#include "module.h"
#include "tools.h"
#include "exchange.h"
#include "cfg.h" 

HardModule::HardModule(void) {
	jdebug("HardModule::HardModule");
	jdebug("_name=%s",_name);
	_ready = FALSE;
//	resetVariables();
	jdebug("end of HardModule's constructor");
}

HardModule::~HardModule() {
}

void HardModule::loadCommonConst(void) {
	jdebug("HardModule: LoadCommonConst");
	_module		= leafDataToChar( deviceCfgFile,_name,_identSection,"Module" );
	_getCRC		= leafDataToChar( deviceCfgFile,_name,"Command","GetCrc" );
	_getStatus	= leafDataToChar( deviceCfgFile,_name,"Command","GetStatus" );
	_getIdent	= leafDataToChar( deviceCfgFile,_name,"Command","GetIdent" );
	_getConst	= leafDataToChar( deviceCfgFile,_name,"Command","GetConst" );
	_reset		= leafDataToChar( deviceCfgFile,_name,"Command","Reset" );

	 const char* id = leafDataToStr( deviceCfgFile,_name,_identSection,"Identification" )
				->data();
	sscanf( id,"%X%X%X%X", &_ident1,&_ident2,&_ident3,&_ident4 );

	_ready = true;
	jdebug("end of HardModule's loadCommonConst");
}

// -------------------------  --------------------------

int HardModule::reset(int dialog=1) {
	jdebug("HardModule::reset");

	resetVariables();

	int retval;
	if( (retval=exch::sendReset(_module,_reset)) != SUCCESS ) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::getConst(int dialog=1) {
	jdebug("HardModule::getConst");
	uchar buf[NCONST];
	int retval;
	if((retval=exch::askData(_module,_getConst,buf,NCONST)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	_const1 = (int)buf[0];
	_const2 = (int)buf[1];
	_const3 = (int)buf[2];
	_const4 = (int)buf[3];
	return SUCCESS;
}

int HardModule::testIdent(int dialog=1) {
	uchar buf[NIDENT];
	int retval;
	if( (retval=exch::askData(_module,_getIdent,buf,NIDENT)) != SUCCESS ) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}

	if( (_ident1 != buf[0]) || (_ident2 != buf[1]) ||
			(_ident3 != buf[2]) || (_ident4 != buf[3]) ) {
		const char* errText = "Invalid Identification!";
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::getValue(uchar& value, uchar getCommand,int dialog=1) {
	jdebug("HardModule::getValue(uchar)");
	int retval;
	if((retval=exch::askByte(_module, getCommand,value)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::getValue(int& value, uchar getCommand,int dialog=1) {
	jdebug("HardModule::getValue(int)");
	int retval;
	if((retval=exch::askWord(_module, getCommand,value)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::sendCommand(uchar command, int dialog=1) {
	int retval;
	if( (retval=exch::sendCommand(_module,command)) != SUCCESS) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::sendValue(uchar command, uchar value, int dialog=1) {
	jdebug("HardModule::sendValue(uchar)");
	int retval;
	if( (retval=exch::sendByte(_module,command,value)) != SUCCESS ) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::sendValue(uchar command, int value, int dialog=1) {
	jdebug("HardModule::sendValue(int)");
	int retval;
	if( (retval=exch::sendWord(_module,command,value)) != SUCCESS ) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::sendValue(uchar command, uchar value1, uchar value2,
														 int dialog=1) {
	jdebug("HardModule::sendValue(value1,value2)");
	uchar buf[4];
	buf[0] = _module;
	buf[1] = command;
	buf[2] = value1;
	buf[3] = value2;
	int retval;
	if( (retval=exch::sendData(buf,4)) != SUCCESS ) {
		const char* errText = exch::getErrTxt(-1*retval);
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

// ---------------------------   ----------

int HardModule::turn(int state,int& currentState,uchar setOn,uchar setOff, 
			int dialog=1,int waitTime=0) { // state = ON or OFF
	jdebug("HardModule::turn");
	if( currentState == state ) return SUCCESS;

	uchar command = setOff;
	if( state == ON ) command = setOn;
	currentState = UNDEFINED;
	if( sendCommand(command,dialog) != SUCCESS ) return FAIL;
	currentState = state;
	if(waitTime && (state==ON)) wait_ms(waitTime);
	return SUCCESS;
}

// class X - int or uchar
template <class X> int HardModule::setValue_(X value, int& currentValue,
				uchar setCommand, uchar getCommand,
				const char* failMessage,
				int dialog=1,
				int waitTime=0) {
	jdebug("HardModule::setValue<X>");
	const char* errText;
	if(value == (X)currentValue) return SUCCESS;
	currentValue = UNDEFINED;

	if( sendValue(setCommand, value, dialog) != SUCCESS ) return FAIL;

	X newValue;
	if( getValue(newValue, getCommand, dialog) != SUCCESS ) return FAIL;
	if(newValue != value) {
		errText = failMessage;
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	currentValue = value;
	if( waitTime && (currentValue != 0) ) wait_ms(waitTime);
	return SUCCESS;
}

int HardModule::setValue(uchar value, int& currentValue,
				uchar setCommand, uchar getCommand,
						const char* failMessage,
						int dialog=1,
						int waitTime=0) {
	jdebug("HardModule::setValue(uchar)");
	return setValue_(value, currentValue, setCommand, getCommand,
									failMessage, dialog, waitTime);
}

int HardModule::setValue(int value, int& currentValue,
				uchar setCommand, uchar getCommand,
						const char* failMessage,
						int dialog=1,
						int waitTime=0) {
	jdebug("HardModule::setValue(int)");
	return setValue_(value, currentValue, setCommand, getCommand,
									failMessage, dialog, waitTime);
}

const int CORRECT_CRC = 0;
const char* BadCRC = "EEPROM corrupted!";

int HardModule::testCRC(int dialog=1) {
	jdebug("HardModule::testCRC");
	uchar crc;
	if( getValue(crc,_getCRC,dialog) != SUCCESS ) return FAIL;

	if(crc != CORRECT_CRC) {
		const char* errText = BadCRC;
		reportError(_identName,errText,dialog);
		return FAIL;
	}
	return SUCCESS;
}

int HardModule::getStatus(uchar& status,int dialog=1) {
	jdebug("HardModule::getStatus");
	return getValue(status,_getStatus,dialog);
}

int HardModule::isMotion(int dialog=1) {
	int retval = exch::sendTest(_module,_testMotion);
	if( retval == SignalACY ) return NO;
// Incahe pust' udet motiion...
	return YES;
/*
	if( retval == SignalACW ) return YES;
// Yesli ne tot signal prishel, to...
	const char* errText = exch::getErrTxt(-1*retval);
	reportError(_identName,errText,dialog);
	return UNDEFINED;
*/
}

int HardModule::stopMotion(int dialog=1) {
	jdebug("HardModule::stopMotion");
	return sendCommand(_stopMotion, dialog);
}
