/*--------------------------------------------------------------------------*/
/** @file tracker.h
@brief  Implementation of interaction (command calls and replies from modules, 
their current data and status keeping, etc.) with autoguide.

                  MASS project: TURBINA module file
                       header file for tracker.cpp



 @author O. Voziakova, Sternberg Institute (ovoz@sai.msu.ru)
 
 @version 2.0
*/
#ifndef TRACKER_H
#define TRACKER_H

#include "module.h"

#define MAX_TRACKER_TEST_INTERVALS	1000	//  5 sec if dev_wait_int = 5 ms

class Tracker : public HardModule {

private:

	double	_maxShift;
	uchar _shiftRA;
	uchar _shiftDec;
	uchar _moveRA;
	uchar _moveDec;
	uchar _forceMotion;

public:
	Tracker(void);
	~Tracker();

	int getConst(void);

	void resetVariables(void);
	void loadConst(void);

	char toSygnByteShift(double shift);

	int shift(int axis, double shift, int dialog=1);	// shift in time-sec
	int smallShift(int axis, double shift, int dialog=1);	// shift in time-sec
	int move(int axis, uchar direction, int dialog=1); 
	int forceMotion(int dialog=1);
};

#endif
