#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <sys/socket.h>
#include <math.h>		// log10
#include "trbsocket.h"
#include "trbexport.h"
#include "tools.h"		// jdebug()

#define SUCCESS	1
#define FAIL	-1

namespace trbsocket {
	struct sockaddr_in name;
	struct sockaddr_in clientName;
	size_t size = sizeof(clientName);
	int mountSocket = -1;
	int clientSocket = -1;
	bool connection = false;

	string::size_type i0 = 0;
}

int trbsocket::init(void) {
	jdebug("trbsocket::init");

	name.sin_family = AF_INET;
	name.sin_port = htons(MOUNT_PORT);
	name.sin_addr.s_addr = htonl(INADDR_ANY);
	int sock = socket(PF_INET,SOCK_STREAM,0);
	if(sock < 0 ) {
		perror("server's socket");
		return FAIL;
	}
//	int flags = fcntl(sock, F_GETFL, 0);
//	fcntl(sock, F_SETFL, flags|O_NONBLOCK);
	if(bind(sock, (struct sockaddr*)&name, sizeof(name)) < 0) {
		perror ("bind");
        return FAIL;
    }
	if(listen(sock, 1) < 0) {
		perror("listen");
		return FAIL;
	}
	mountSocket = sock;
	atexit(closeSocket);
	return SUCCESS;
}

void trbsocket::closeSocket(void) {
	jdebug("trbsocket::closeSocket");
 	if(mountSocket > 0) if(close(mountSocket) < 0) perror("close mountSocket");
	if(clientSocket > 0) if(close(clientSocket) < 0) perror("close clientSocket");
}

int trbsocket::newConnection(void) {
	jdebug("trbsocket::newConnection");

	int newSock = accept(mountSocket, (struct sockaddr *)&clientName, &size);
	if(newSock < 0) {
//		if(errno == EAGAIN) return FAIL;
		perror ("accept");
		return FAIL;
	}
//	int flags = fcntl(newSock, F_GETFL, 0);
//	fcntl(newSock, F_SETFL, flags|O_NONBLOCK);
	clientSocket = newSock;
	printf("Connect from host %s, port %hd.\n",
		inet_ntoa(clientName.sin_addr), ntohs(clientName.sin_port));
	connection = true;
	return(newSock);
}

int trbsocket::extractInstruction(int i) {
	i0 = string::size_type(i);
	return 0;
}

int trbsocket::extractInstruction(string from, string& instruction, int& comid) {
	string::size_type i1 = from.find("\n",i0);
	if(i1 == string::npos) return 0;

	int i_comid = from.find(" ",i0);
	string comid_str = from.substr(i0,i_comid-i0);
	instruction = from.substr(i_comid+1,i1-i_comid-1);
	comid = atoi(comid_str.c_str());
	printf("i0=%d i1=%d i_comid=%d comid_str=|%s| comid=%d instruction=|%s|\n",
				i0,i1,i_comid,comid_str.c_str(),comid,instruction.c_str());

	i0 = i1+1;
	return i1;
}

void trbsocket::extractKeyword(string* intruction, string& keyword) {
	string::size_type i1 = intruction->find(" ",0);
	if(i1 == string::npos) {
		printf("keyword == instruction!!!\n");
		keyword = *intruction;
	} else keyword = intruction->substr(0,i1);
	return;
}

int trbsocket::getParam(string* instruction, const char* paramName, string& paramValue) {
	string paramName1 = paramName;
	paramName1 += "=";
	string::size_type i1 = instruction->find(paramName1,0);
	if(i1 == string::npos) {
		printf("param |%s| in |%s| not found!\n",paramName,instruction->c_str());
		return FAIL;
	}
	string::size_type i11 = instruction->find("=",i1);
	if(i11 == string::npos) {
		printf("param |%s| in |%s| not found!\n",paramName,instruction->c_str());
		return FAIL;
	}
	string::size_type i2 = instruction->find(" ",i11+1);
	if(i2 == string::npos) paramValue = instruction->substr(i11+1);
	else paramValue = instruction->substr(i11+1,i2-i11-1);
	printf("|%s| = |%s|\n",paramName, paramValue.c_str());
	return SUCCESS;
}

const int max_length = 80;

int trbsocket::getCommand(void) {
	char str[max_length];
	int len = recv(clientSocket,str,max_length,0);
	if(len <= 0) {
		perror("server: recv");
		connection = false;
		close(clientSocket);
		clientSocket = -1;
		return FAIL;
	}
	str[len] = '\0';
	string instruction;
	extractInstruction(0);
//	printf("from client:|%s|\n",str);
	int comid=0;
	while( extractInstruction(str,instruction,comid) ) {

//		printf("client> comid=%d |%s|\n",comid,instruction.c_str());

		string keyword;
		extractKeyword(&instruction, keyword);

		if(keyword == "QUIT") {
			SVexport::setQuitProgram(comid);
			continue;
		}
		else if(keyword == "STOP") {
			SVexport::setStopScenario(comid);
			continue;
		}
		else if(keyword == "BREAK") {
			SVexport::setStopCounting(comid);
			continue;
		}
		else if(keyword == "RUN") {
			SVexport::setPlayScenario(comid);
			continue;
		}
		else if(keyword == "STAR") {
			string hr;
			getParam(&instruction,"HR",hr);
			SVexport::setNewCurrentStar(comid,hr.c_str());
			continue;
		}
		else sendError(comid);
	}
	return SUCCESS;
}

int trbsocket::sendToSV(int comid, const char* mess) {
// lll
//	const int max_comid_len = int(floor(log10(MAX_COMID))+2);
	char num[max_comid_len];
	sprintf(num,"%d ",comid);
	string toSV = num;
	toSV += mess;
	printf("trbsocket::sendToSV>|%s|\n",toSV.c_str());
	if( send(clientSocket,toSV.c_str(),strlen(toSV.c_str()),0) <= 0 ) {
		perror("server: send");
		return FAIL;
	}
	return SUCCESS;
}

int trbsocket::sendOK(int comid,int waitsec) {
	char toSV[80];
	if(waitsec) {
		sprintf(toSV,"OK WAIT=%d\n",waitsec);
		return sendToSV(comid, toSV);
	} else return sendToSV(comid, "OK\n");
}

int trbsocket::sendError(int comid) {
//	printf("trbsocket::sendError\n");
	return sendToSV(comid,"ERROR\n");
}

int trbsocket::socket_cycle(void) {
	jdebug("trbsocket::socket_cycle");
	while(1) {
		if(connection) {
			getCommand();
//			if( getCommand() == QUIT ) return SUCCESS;
		} else newConnection();
//		sleep(1);
	}
}
